@@ -7,11 +7,11 @@ Users looking for something as simple to use as libmilter should take a look at
 
 from typing import TYPE_CHECKING
 
-from .core import FilterProtocol
+from .core import FilterProtocol as FilterProtocol
 from .exceptions import *
 from .messages import *
 
 if TYPE_CHECKING:
-	from .buffer import FixedSizeBuffer
+	from .buffer import FixedSizeBuffer as FixedSizeBuffer
 
-__version__ = "0.1.2"
+__version__ = "0.1.3"
