@@ -17,6 +17,7 @@ from __future__ import annotations
 from abc import ABCMeta
 from abc import abstractmethod
 from collections.abc import Collection
+from collections.abc import Iterable
 from collections.abc import Iterator
 from collections.abc import Mapping
 from dataclasses import dataclass
@@ -41,7 +42,7 @@ if TYPE_CHECKING:
 LONG = Struct("!L")
 
 __all__ = [
-	"Family", "Stage",
+	"Family", "Stage", "BitField", "ActionFlags", "ProtocolFlags",
 	"Message", "Negotiate", "Macro", "Connect", "Helo", "EnvelopeFrom",
 	"EnvelopeRecipient", "Data", "Unknown", "Header", "EndOfHeaders", "Body",
 	"EndOfMessage", "Abort", "Close", "Continue", "Reject", "Discard", "Accept",
@@ -110,6 +111,83 @@ class Stage(int, Enum):
 	END_HEADERS = 6
 
 
+BFSelf = TypeVar("BFSelf", bound="BitField")
+
+
+class BitField(int, Enum):
+	"""
+	Base class for bit-field enums like ActionFlags and ProtocolFlags
+	"""
+
+	@classmethod
+	def pack(cls: type[BFSelf], flags: Iterable[BFSelf]) -> int:
+		rflag = 0
+		for flag in flags:
+			rflag |= flag
+		return rflag
+
+	@classmethod
+	def unpack(cls: type[BFSelf], bitfield: int) -> set[BFSelf]:
+		return {flag for flag in cls if flag & bitfield}
+
+
+class ActionFlags(BitField):
+	"""
+	Bit-field values for the `Negotiate.action_flags` field of the `Negotiate` message
+
+	The values correspond to the `SPFIF_*` codes as described in
+	https://pythonhosted.org/pymilter/milter_api/smfi_register.html#flags
+	"""
+
+	ADD_HEADERS = ADDHDRS = 0x1
+	CHANGE_HEADERS = CHGHDRS = 0x10
+	CHANGE_BODY = CHGBODY = 0x2
+	ADD_RECIPIENT = ADDRCPT = 0x4
+	ADD_RECIPIENT_PAR = ADDRCPT_PAR = 0x80
+	DELETE_RECIPIENT = DELRCPT = 0x8
+	QUARANTINE = 0x20
+	CHANGE_FROM = CHGFROM = 0x40
+	SETSYMLIST = 0x100
+
+
+class ProtocolFlags(BitField):
+	"""
+	Bit-field values for the `Negotiate.protocol_flags` field of the `Negotiate` message
+
+	The values correspond to the `SMFIP_*` codes described in
+	https://pythonhosted.org/pymilter/milter_api/xxfi_negotiate.html
+	"""
+
+	NO_CONNECT = 0x1
+	NO_HELO = 0x2
+	NO_SENDER = NO_MAIL = 0x4
+	NO_RECIPIENT = NO_RCPT = 0x8
+	NO_BODY = 0x10
+	NO_HEADERS = NO_HDRS = 0x20
+	NO_END_OF_HEADERS = NO_EOH = 0x40
+	NO_UNKNOWN = 0x100
+	NO_DATA = 0x200
+
+	SKIP = 0x400
+
+	REJECTEDT_RECIPIENT = RCPT_REJ = 0x800
+
+	NR_CONNECT = NR_CONN = 0x1000
+	NR_HELO = 0x2000
+	NR_SENDER = NR_MAIL = 0x4000
+	NR_RECIPIENT = NR_RCPT = 0x8000
+	NR_DATA = 0x10000
+	NR_UNKNOWN = NR_UNKN = 0x20000
+	NR_END_OF_HEADERS = NR_EOH = 0x40000
+	NR_BODY = 0x80000
+	NR_HEADER = NR_HDR = 0x80
+
+	HEADER_LEADING_SPACE = HDR_LEADSPC = 0x100000
+
+	MAX_DATA_SIZE_256K = MDS_256K = 0x10000000
+	MAX_DATA_SIZE_1M = MDS_1M = 0x20000000
+
+
 class Message(metaclass=ABCMeta):
 	r"""
 	A base class for all messages which also handles packing and unpacking messages
@@ -171,7 +249,7 @@ class Message(metaclass=ABCMeta):
 	@classmethod
 	def unpack(cls, buf: FixedSizeBuffer) -> tuple[Message, int]:
 		"""
-		Unpack the first message in a buffer and return an instance of the correct class
+		Unpack a message in buff and return a subclass instance and the number of bytes read
 		"""
 		hdr_size = cls._hdr_struct.size
 		if buf.filled < hdr_size:
@@ -269,13 +347,18 @@ class BytesMessage(Message):
 	content: memoryview
 
 	def __init__(self, content: bytes):
-		self.content = memoryview(content)
+		self.content = memoryview(content).toreadonly()
 
 	def __repr__(self) -> str:
 		content = repr(self.content.tobytes()) if len(self.content) <= 30 else \
 			f"{self.content[:20].tobytes()!r} + {len(self.content)-20} further bytes"
 		return f"{self.__class__.__name__}({content})"
 
+	def __eq__(self, other: object) -> bool:
+		if not isinstance(other, type(self)):
+			return NotImplemented
+		return other.content == self.content
+
 	@classmethod
 	def from_buffer(cls: type[Self], buf: memoryview) -> Self:
 		return cls(buf)
@@ -492,7 +575,7 @@ class EnvelopeRecipient(Message, ident=b"R"):
 		Self = TypeVar("Self", bound="EnvelopeRecipient")
 
 	recipient: bytes
-	arguments: list[bytes]
+	arguments: list[bytes] = field(default_factory=list)
 
 	@classmethod
 	def from_buffer(cls: type[Self], buf: memoryview) -> Self:
@@ -594,26 +677,31 @@ class Continue(NoDataMessage, ident=b"c"):
 	A filter response instructing an MTA to continue without a specific decision
 	"""
 
+
 class Reject(NoDataMessage, ident=b"r"):
 	"""
 	A filter response instructing an MTA to reject a message
 	"""
 
+
 class Discard(NoDataMessage, ident=b"d"):
 	"""
 	A filter response instructing an MTA to reject a message without informing the client
 	"""
 
+
 class Accept(NoDataMessage, ident=b"a"):
 	"""
 	A filter response instructing an MTA to accept a message
 	"""
 
+
 class TemporaryFailure(NoDataMessage, ident=b"t"):
 	"""
 	A filter response instructing an MTA to reject a message temporarily (i.e. with a 4xx)
 	"""
 
+
 class Skip(NoDataMessage, ident=b"s"):
 	"""
 	A filter response instructing an MTA to skip the rest of a message's content
@@ -629,7 +717,6 @@ class ReplyCode(BytesMessage, ident=b"y"):
 	"""
 
 
-
 # Modification Messages
 
 @dataclass
